'\" t
.\" Copyright 1999 Oracle and/or its affiliates. All rights reserved.
.\"
.\" Permission is hereby granted, free of charge, to any person obtaining a
.\" copy of this software and associated documentation files (the "Software"),
.\" to deal in the Software without restriction, including without limitation
.\" the rights to use, copy, modify, merge, publish, distribute, sublicense,
.\" and/or sell copies of the Software, and to permit persons to whom the
.\" Software is furnished to do so, subject to the following conditions:
.\"
.\" The above copyright notice and this permission notice (including the next
.\" paragraph) shall be included in all copies or substantial portions of the
.\" Software.
.\"
.\" THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
.\" IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
.\" FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
.\" THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
.\" LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
.\" FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
.\" DEALINGS IN THE SOFTWARE.
.\"
.TH XkbAllocControls __libmansuffix__ __xorgversion__ "XKB FUNCTIONS"
.SH NAME
XkbAllocControls \- Allocates an XkbControlsRec structure in the XkbDescRec
.SH SYNOPSIS
.HP
.B Status XkbAllocControls
.BI "(\^XkbDescPtr " "xkb" "\^,"
.BI "unsigned int " "which" "\^);"
.if n .ti +5n
.if t .ti +.5i
.SH ARGUMENTS
.TP
.I \- xkb
Xkb description in which to allocate ctrls rec
.TP
.I \- which
mask of components of ctrls to allocate
.SH DESCRIPTION
.LP
The need to allocate an XkbControlsRec structure seldom arises; Xkb creates one 
when an application calls 
.I XkbGetControls 
or a related function. For those situations where there is not an XkbControlsRec 
structure allocated in the XkbDescRec, allocate one by calling 
.I XkbAllocControls.

.I XkbAllocControls 
allocates the 
.I ctrls 
field of the 
.I xkb 
parameter, initializes all fields to zero, and returns Success. If the 
.I ctrls 
field is not NULL, 
.I XkbAllocControls 
simply returns Success. If 
.I xkb 
is NULL, 
.I XkbAllocControls 
reports a BadMatch error. If the 
.I ctrls 
field could not be allocated, it reports a BadAlloc error.

The 
.I which 
mask specifies the individual fields of the 
.I ctrls 
structure to be allocated and can contain any of the valid masks defined in 
Table 1. 
(SHOULD THIS COMMENT BE LEFT IN????)
Because none of the currently existing controls have any structures associated 
with them, which is currently of little practical value in this call.

Table 1 shows the actual values for the individual mask bits used to select 
controls for 
modification and to enable and disable the control. Note that the same mask bit 
is used to 
specify general modifications to the parameters used to configure the control 
(which), and to 
enable and disable the control (enabled_ctrls). The anomalies in the table (no 
"ok" in column) 
are for controls that have no configurable attributes; and for controls that are 
not boolean 
controls and therefore cannot be enabled or disabled.

.TS
c s s s
l l l l
l l l l
l l l l. 
Table 1 Controls Mask Bits
_
Mask Bit	which or	enabled	Value
	changed_ctrls	_ctrls
_
XkbRepeatKeysMask	ok	ok	(1L<<0)
XkbSlowKeysMask	ok	ok	(1L<<1)
XkbBounceKeysMask	ok	ok	(1L<<2)
XkbStickyKeysMask	ok	ok	(1L<<3)
XkbMouseKeysMask	ok	ok	(1L<<4)
XkbMouseKeysAccelMask	ok	ok	(1L<<5)
XkbAccessXKeysMask	ok	ok	(1L<<6)
XkbAccessXTimeoutMask	ok	ok	(1L<<7)
XkbAccessXFeedbackMask	ok	ok	(1L<<8)
XkbAudibleBellMask		ok	(1L<<9)
XkbOverlay1Mask		ok	(1L<<10)
XkbOverlay2Mask		ok	(1L<<11)
XkbIgnoreGroupLockMask		ok	(1L<<12)
XkbGroupsWrapMask	ok		(1L<<27)
XkbInternalModsMask	ok		(1L<<28)
XkbIgnoreLockModsMask	ok		(1L<<29)
XkbPerKeyRepeatMask	ok		(1L<<30)
XkbControlsEnabledMask	ok		(1L<<31)
XkbAccessXOptionsMask	ok	ok	(XkbStickyKeysMask | 
			XkbAccessXFeedbackMask)
XkbAllBooleanCtrlsMask		ok	(0x00001FFF) 
XkbAllControlsMask	ok		(0xF8001FFF)
.TE
.nh
.SH "RETURN VALUES"
.TP 15
SUCCESS
allocates the ctrls field of the xkb parameter, initializes all fields to zero
.sp
.in 20
the ctrls field is not NULL
.SH DIAGNOSTICS
.TP 15
.B BadAlloc
Unable to allocate storage
.TP 15
.B BadMatch
A compatible version of Xkb was not available in the server or an argument has 
correct type and range, but is otherwise invalid
.SH "SEE ALSO"
.BR XkbGetControls (__libmansuffix__)
